import processing.video.*;
import java.io.File;

Capture cam;
int camWidth, camHeight;

void setup()
{
  frameRate(30);
  size(1280, 720);
  
  String[] cameras = Capture.list();
  if (cameras.length == 0) exit();
  cam = new Capture(this, cameras[0]);
  cam.start();
  
  //
  imgDrop = loadImage("drop.png");
  
  //
  InitBGM();
  //
  InitCats();
  
  DeleteScreenShot();
}

void DeleteScreenShot()
{
  String fileName = sketchPath("data/temp/still_cut.tif");
  File f = new File(fileName);
  if (f.exists()) {
    f.delete();
  }
  
}

void InitBGM(){
  bgm.file = new SoundFile[num_of_BGM];
  for(int i=0; i<bgm.file.length; i++){
    bgm.file[i] = new SoundFile(this, "sounds/bgm" + (i+1) + ".mp3");
  }
}

int ww = 320;
int hh = 240;

PGraphics pg;
color[][] backgrounds = new color[10][ww*hh];
color[] background = new color[ww*hh];
void init_background()
{
  float r, g, b;
  for (int x = 0; x < ww; x++)
    for (int y = 0; y < hh; y++) {
      r = 0;
      g = 0;
      b = 0;
      for (int i = 0; i < 10; i++) {
        r += red(backgrounds[i][x+y*ww]);
        g += green(backgrounds[i][x+y*ww]);
        b += blue(backgrounds[i][x+y*ww]);
      }
      r /= 10;
      g /= 10;
      b /= 10;
      background[x+y*ww] = color(r, g, b);
    }
}

boolean changed(int x, int y, int dx, int dy)
{
  if ((x + dx < 0 || ww <= x + dx) || (y + dy < 0 || hh <= y + dy)) return false;
  return
    abs(red(pg.pixels[x+y*ww]) - red(background[(x+dx)+(y+dy)*ww]))
    + abs(green(pg.pixels[x+y*ww]) - green(background[(x+dx)+(y+dy)*ww]))
    + abs(blue(pg.pixels[x+y*ww]) - blue(background[(x+dx)+(y+dy)*ww])) >= 30;
}

boolean changed_rad(int x, int y)
{
  int rad = 4;
  boolean flag = true;
    for (int dx = -rad; dx <= rad; dx++)
      for (int dy = -rad; dy <= rad; dy++)
        flag = flag && changed(x, y, dx, dy);
  return flag;
}

int step = 0; //0: pre-time, 1:play-time
void mousePressed()
{
  if(step==0) step = 1;
}

int count = 0;
void draw()
{
  if(!cam.available()) return;
  cam.read();
  
  //pre-time
  if(step==0){
    translate(width,0);
    scale(-1,1);
    image(cam,0,0,width,height);    
    return;
  }
  
  //play-time
  pg = createGraphics(ww,hh);
  pg.beginDraw();
  pg.image(cam,0,0,ww,hh);
  pg.loadPixels();
  
  if (count < 10) {    
    pg.endDraw();
    
    for (int x = 0; x < ww; x++)
      for (int y = 0; y < hh; y++)
        backgrounds[count][x+y*ww] = pg.pixels[x+y*ww];
  }
  else {
    if (count == 10) init_background();
    colorMode(HSB, 100);
    for(int x=0; x<ww; x++)
      for(int y=0; y<hh; y++)
        if(changed_rad(x, y)){
          float t = random(0,1);
          
          color col = color(random(60,60), 80*(1-t)+10, 20*t+80); //(count % 10 < 5)? color(247, 202, 201) : color(146, 168, 209); 
          pg.pixels[x + y*ww] =  color(10,10,10);
          if(random(0,100) < 3){
            MakeDrop((float)x * width/ww, (float)y * height/hh, col); 
          }
        }
    colorMode(RGB, 255);
    pg.updatePixels();
    pg.endDraw();
    
    pushMatrix();
    translate(width,0);
    scale(-1,1);
    image(pg,0,0,width,height);
    //popMatrix();
    
    //drops
    for(int i=0; i<drops.size(); i++){
      drops.get(i).Draw(); 
    }
    
    
    
    //BG & water
    bg.DrawLayer1();
    //cat
    DrawCats();
    //BG & water
    waterLine.Draw();
    bg.DrawLayer2();
    popMatrix();
  }
  count++;
  
  
  if(!SomeCatOutOfScreen())
  {
    bgm.Loop(1);
    //timer
    if(!timer.is_initialized) timer.Init();
    timer.Update();
    
    //draw TextTime;
    fill(255);
    textAlign(RIGHT, UP);
    textSize(width*0.03);
    text(""+(timer.total_millis/1000)+" sec", width*0.95, height*0.06);
  }
  else
  {
    if(!AllCatOutOfScreen()){      
      bgm.Loop(2);
    }
    //timer
    timer.UpdateWithoutTotalMillis();
    
    //draw ending text
    textSize(width*0.07);
    textAlign(CENTER, CENTER);
    fill(0);
    text("Failed at\n"+(timer.total_millis/1000)+" sec", width*0.5 + 5, height*0.4 + 5);
    fill(255);
    text("Failed at\n"+(timer.total_millis/1000)+" sec", width*0.5, height*0.4);
    
    
    if(AllCatOutOfScreen()){
      if(!is_last_cut_pictured){
        ScreenShot(); 
        is_last_cut_pictured = true;
      }
      if(img_last_cut == null){
        img_last_cut = loadImage("temp/still_cut.tif");
      }else{      
        image(img_last_cut, 0, 0, width, height); 
      }
      bgm.Loop(3);
    }
  }
}
boolean is_last_cut_pictured = false;
PImage img_last_cut = null;

void ScreenShot(){
  colorMode(HSB,100);
  loadPixels();
  for(int x=0; x<width; x++){
    for(int y=0; y<height; y++){
      color pc = pixels[x + y*width]; 
      pixels[x+y*width] = color(hue(pc), 0 , brightness(pc));
    }
  }
  colorMode(RGB,255);
  updatePixels();
  save("data/temp/still_cut");  
}

void MakeDrop(float x, float y, color col)
{
  Drop new_drop = new Drop(x,y,col);
  drops.add(new_drop);
}

void InitCats()
{
  for(int i=0; i<num_of_cats; i++){
    float cat_width = 0.9/num_of_cats * width;
    float x_init = cat_width*((float)i+0.5) + width*0.05;
    Cat new_cat = new Cat(x_init);
    cats.add(new_cat); 
  }
}
void DrawCats()
{
   for(int i=0; i<cats.size(); i++){
     cats.get(i).Draw(); 
   }
}

boolean SomeCatOutOfScreen()
{
  if(cats.get(0).x <= 0) return true;
  else return false;
}
boolean AllCatOutOfScreen()
{
  for(int i=0; i<cats.size(); i++){
    if( cats.get(i).x > - cats.get(0).size_factor * width ) return false;
  }
  return true;
}